# List of Changes for HZDR Multiphase Addon for OpenFOAM

## [Latest Release]

### Bugfix

- fix post processing of dropletEntrainmentByShear test case
- Fix path to online repository for Doxygen
- alphaMax moved to phaseProperties for flotation tutorials
- Allow control dict update when using HZDRCourantNo

### Documentation

- Update QT dependencies
- More descriptive merge request template for releases
- Installation instructions for Mac OS
- Modify command for running megalinter locally such that container is removed upon exit
- Update domain to GitLab instance
- beautify shortcut table

### Misc

- Remove configuration files for flotation cases and switch to SchillerNaumann drag model
- Replaced prefix "HZDR" for libraries and exucutables by the more general "addon"
- Make display support for docker images optional
- Correct sprint cycle bash script to make linter happy
- Fixed formatting of blockMeshDictCylinder template
- Update Snakemake version for new workflow design
- Adapt VS Code instructions
- Provide VS Code settings as files and add instructions to CONTRIBUTING.md
- Remove checkbox for template repository

### MultiMorph Model

- Rename OpenFOAM-Hybrid to MultiMorph Model

### OpenFOAM

- Remove foamyHexMesh

### OpenFOAM-Hybrid

- add description of addon specific changes to header of ...
- write shear based under-resolution indicator at start time

### Physical Modelling

- Added drag models of Tomiyama et al. (1998) for pure and slightly contaminated systems
- Entrainment model revision
- Remove not working freeSurface mesh motion solver
- Make degassingPhases fvModel compatible with blending framework

### Population Balance

- Added GPU-accelerated version of the coalescence model of Lehr et al. (2002) that works with the GPU-accelerated source term assembly
- Enabled runtime selection of GPU-accelerated binary breakup models in combination with CUDApopulationBalance
- Enabled runtime selection of GPU-accelerated coalescence models in combination with CUDApopulationBalance
- Introduced automatic assignment of MPI ranks to one or more GPU devices
- Added GPU-accelerated source term assembly for breakup, binary breakup and drift
- Some improvements to GPU-accelerated coalescence source term assembly
- Add test cases for (binary)Breakup and drift for testing the GPU-accelerated source term assembly
- First working GPU-accelerated coalescence source term assembly
- Added basic code structure for implementing the GPU-acceleration of the source term assembly using NVIDIA CUDA

### Post-Processing

- add caseDict volAverage

### Test

- revise Ma entrainment test case to be more standard OF-Hybrid
- Added HZDR header to Alltest script to mark it as an extension of the original script supplied by OpenFOAM
- Add -configure option to Alltest that only prepares setups to run a single time step but skips execution

### Tutorial

- sync hybrid cases among repositories

---

## [HZDR-Multiphase-Addon-10-s.1-hzdr.2] - 2022-10-19

### Documentation

- Clarify that Debian, Docker and compiling from sources are exclusive installation options
- Add references to README.md

---

## [HZDR-Multiphase-Addon-10-s.1-hzdr.1] - 2022-10-14

### Bugfix

- use unquoted value to allow for proper command line argumens generation
- functionObject deviation now properly writes out deviation as field as well as in log file
- wrong apt command in dockerfile
- define arguments of foamShellRunner with '--' preventing abbreviations
- for degassingPhases fvModel add sink terms for transported variables
- qt5-default package is not available in Ubuntu 21 and newer

### Documentation

- Make sure to remove all hyphens from docker compose commands
- Source OpenFOAM environment before applying sprint cycle patches
- Fix typo in installation instructions
- Re-structure file headers with better information about copyright
- Modified header with improved copyright
- Refer to nightly build addon Debian package in installation instructions
- correct latex formulas and remove problematic extra package to fix Doxygen formula rendering
- Automatically generate changelog file based on git commits
- align keywords in Gitlab, Rodare, codemeta.json  and HIFIS Software Spotlight
- Correct Email address of Mazen Draw in codemeta.json
- add position information to contact details
- Add milestone information to support request issue template
- New issue template for internal and external support requests
- Fix broken links in doxygen
- correct typo in installation instruction
- add information to installation instructions that adding the local ssh key to the gitlab account is required for cloning the repository
- Correct non-compliant file headers
- Correct statement on supported Ubuntu versions
- Add Tingting Zhang as contributor

### Misc

- Add words to spell checker list
- Refine gitignore rules with respect to postProcessing folders
- Refine gitignore rules related to postProcessing folders
- Enable various models and utilities for release
- Adapt VS Code configuration script to meet linter requirements
- Include recommended VS Code extensions in configuration script
- Fix typo in VS Code settings
- Let VS Code classify C and H files as C++ independently from any extension
- update VS Code settings to disable recommendations and clean-up
- Do not hide .git* in VS Code file explorer (.git folder is hidden anyway)
- Remove tar.gz and tgz from gitignore
- minor corrections in createVSCodeProject.sh
- writeCellCoords functionObject to print the cell-centre coodinates for a given cell id
- Removed duplicate category from .gitignore
- allow tracking of folders with leading digits, only ignore names consisting exclusively of digits and decimal points

### OpenFOAM

- Sprint Cycle Upgrade 10-s.1

### OpenFOAM-Hybrid

- protect dominant continuous phase from disintegration
- simplify user access to curvature
- prevent disintegration of dominant continuous phase

### Physical Modelling

- add kOmegaSSTSatoIt turbulence model
- Implement droplet entrainment model
- Add low-Re correction to Shi wallLubrication model
- Write mass rate only once for master

### Post-Processing

- Added multiphaseEulerFoam-specific functionObject for extracting dimensionless numbers

### Test

- Counter-current test for resolved interface drag
- Correct exit code of foamShellRunner.py upon failure
- add test and demo case for disintegrationToDisperse morphology transfer
- Simplified commands for obtaining directory basename in Allrun scripts
- Refactor functionObject deviation

---

## [HZDR-Multiphase-Addon-9-s.1-hzdr.2] - 2022-06-21

### Bugfix

- Add output to doxygen to check sprint version detection
- correct path for cleaning bot
- Fix wrong variable name in gitlab deploy job for debian package
- fix deployment pipeline for openfoam debian packages

### Documentation

- Substituted remaining apt-get in installation instructions
- added missing QtCharts package to installation instructions
- fix headers of HZDR dictionary files

### Misc

- replaced m4-based template blockMesh dictionaries using #calc and #codeStream
- update and unify gitignore

### OpenFOAM-Hybrid

- Write mass transfer rates for all morphology transfer phaseTransfer models
- re-formulation of turbulent dispersion force to make it consistent and more stable
- unify default hybrid setups with caseDicts

### Physical Modelling

- Updated SaffmanMei lift model implementation following its contribution to OpenFOAM-dev
- Updated Kusters breakup model implementation following its contribution to OpenFOAM-dev
- Updated AdachiStuartFokkink agglomeration model implementation following its contribution to OpenFOAM-dev
- Added basic infrastructure for an iterative drag and non-drag force correction

### Post-Processing

- generally use CSV as output format

### Test

- added -from and -to options to Alltest
- Fix and simplify some cases regarding post-processing and plotting

### Tutorial

- Added case simulating solid particle agglomeration and breakage

---

## HZDR-Multiphase-Addon-9-s.1-hzdr.1 - 2022-04-26

### Baseline

- swarm correction model to blend out drag and lift model
- breakup and coalescence case dictionaries and degassing boundary condition for bubble columns

### Bugfix

- Update sizeDistribution caseDict to British English
- Cross-project pipeline is now allowed to fail
- patch does not contain any data

### Documentation

- update license information to GPL v3 or later
- add information about clone for docker compose
- correct sprint cycle commit sha for OpenFOAM-dev and ThirdParty-dev
- add links for further information about syntax of Compose files
- add missing docker login registry.hzdr.de
- Correct installation instructions with respect to WM_PROJECT_SITE_DIR
- Added information on git-lfs to installation instructions
- Add doi to codemeta and openfoam website link to Readme.md
- Split command line commands into blocks for efficient use of "copy" button from the browser
- reference to OpenFOAM code quality guide
- New checklist for merge requests containing a possible release contribution
- Modify cite us as for processing with pandoc to pdf
- New installation instructions
- Centred the Class Declaration comment

### OpenFOAM

- Rationalized the naming of scalar limits

### Physical Modelling

- Shi et al (2021) wall lubrication model for rigid spheres

### Population Balance

- functionObject for computing integral and mean properties of a size distribution
- Cleanup of unnecessary includes files
- Prepare coalescence and breakup models of Liao et al. (2015) for contribution
- Correct Ceff coefficient in Liao coalescence model implementation
- CUDA CoulaloglouTavlarides kernel
- Configurable number of CUDA streams
- add reference solution to test cases
- Cuda Test for LehrMilliesMewes kernel
- CUDA LehrMilliesMewes binary breakup model

### Post-Processing

- Python script for calculating time derivative of time series

### Test

- set terminal to color in createGraph scripts of population balance tests

---
automatically generated with [git-chglog](https://github.com/git-chglog/git-chglog)
