# Installation Instructions for HZDR Multiphase Addon for OpenFOAM

[[_TOC_]]

## 1. General remarks

The installation instructions will use the following environment variables

- `WM_PROJECT_DIR`: directory, where OpenFOAM source code provided by The
  OpenFOAM Foundation is installed
- `WM_PROJECT_USER_DIR`: OpenFOAM user directory, where user's code or addons
  can be installed
- `WM_PROJECT_SITE`: OpenFOAM site directory, where addons can be installed
- `WM_PROJECT_VERSION`: OpenFOAM version, e.g., dev, 6, 9, 10-s.1

## 2. Option 1: Installation of Debian Packages

Two debian packages are provided for Ubuntu Focal Fossa (20.04) or newer:

- `openfoam-10-s.1-amd64.deb`: installs the required OpenFOAM release in
  `/opt/OpenFOAM`
- `hzdr-multiphase-addon-for-openfoam-10-s.1-nightly-amd64.deb`: installs the
  HZDR Multiphase Addon for OpenFOAM in `/opt/OpenFOAM/site`

For installation download both packages from `Packages & Registries`[^1] menu
and run

```shell
sudo apt update
sudo apt install ./openfoam-10-s.1-amd64.deb
sudo apt install ./hzdr-multiphase-addon-for-openfoam-10-s.1-hzdr.3-amd64.deb
```

once the installation is finished add

```shell
source /opt/OpenFOAM/OpenFOAM-10-s.1/etc/bashrc
```

to your `~/.bashrc`.

Optionally, install Paraview

```shell
sudo apt install paraview
```

and use

```shell
paraFoam -builtin
```

to run Paraview.

## 3. Option 2: Compiling from sources

### 3.1. Prerequisites and Source Code for OpenFOAM

To compile the HZDR Multiphase Addon for OpenFOAM an installation of OpenFOAM
provided by The OpenFOAM Foundation[^2] is required. The source code is
available at the OpenFOAM-dev[^3] repository at GitHub.

Note, if you have downloaded the HZDR Multiphase Addon for OpenFOAM from
Rossendorf Data Repository (RODARE) an archive with the required commit of
the OpenFOAM release source code is available for download. Obtaining the source
code from GitHub or the provided archive is equal, instead of cloning the
repository unpack the archive to the required folder and follow the
instructions.

On an Ubuntu Linux system install the following general packages for OpenFOAM

```shell
sudo apt install build-essential flex bison git-core cmake zlib1g-dev \
     libboost-system-dev libboost-thread-dev libopenmpi-dev openmpi-bin \
     gnuplot libreadline-dev libncurses-dev libxt-dev ca-certificates libfl-dev
```

Create a new directory for OpenFOAM

```shell
mkdir -p $HOME/OpenFOAM
cd $HOME/OpenFOAM
```

and enter the newly created directory. Now, clone the ThirdParty sources from
GitHub and reset the sources to the required commit sha

```shell
git clone https://github.com/OpenFOAM/ThirdParty-dev.git ThirdParty-10-s.1
(cd ThirdParty-10-s.1 && git reset --hard 1aa9e0871798ee3deb69ea59235fcd2ec2a26736)
```

The same has to be done for the OpenFOAM sources, clone from GitHub repository
and reset to the required commit sha

```shell
git clone https://github.com/OpenFOAM/OpenFOAM-dev.git OpenFOAM-10-s.1
(cd OpenFOAM-10-s.1 && git reset --hard 380558649e1feb5355d746a50f73d22b6fda5e3c)
```

Once all repositories are successfully cloned and reset, configure and load
OpenFOAM environment

```shell
sed -i -e "s/\(WM_PROJECT_VERSION=\).*$/\110-s.1/" OpenFOAM-10-s.1/etc/bashrc
sed -i -e "s/\(WM_PROJECT_VERSION=\).*$/\110-s.1/" OpenFOAM-10-s.1/etc/cshrc
source $HOME/OpenFOAM/OpenFOAM-10-s.1/etc/bashrc
```

**Mac OS:** As a default Mac OS uses the Apple File System (APFS) and this is
not case-sensitive. Before cloning any source code a new volume with
case-sensitive APFS is required[^16].

### 3.2. Prerequisites and Source Code for HZDR Multiphase Addon

There are two possible ways to obtain the source code for the HZDR Multiphase
Addon for OpenFOAM:

- **Helmholtz Code Base**[^1]: For Helmholtz and Friends via Helmholtz AAI[^5]
  or using a HZDR guest account
- **Rossendorf Data Repository (RODARE)**[^6]: For everybody

For usage of the GPU (graphic processor unit) accelerated population balance
method a NVIDIA GPU with CUDA support is required as well as an installation of
the NVIDIA Cuda Toolkit[^4] with the NVCC compiler.

On an Ubuntu Linux system the following additional packages are required

```shell
sudo apt install qt5-qmake qtchooser qtbase5-dev libqt5charts5-dev
```

#### 3.2.1 Source Code from Helmholtz Code Base

For cloning the source code of the HZDR Multiphase Addon for OpenFOAM, first
add the public ssh key[^7] of your local machine to your Gitlab account.

The HZDR Multiphase Addon[^1] repository includes content that is versioned
using `git-lfs`[^8]

```shell
sudo apt install git-lfs
```

to store large binary files in the repository.

Then create a directory for OpenFOAM user developments and clone the
addon

```shell
source $HOME/OpenFOAM/OpenFOAM-10-s.1/etc/bashrc
mkdir -p $WM_PROJECT_USER_DIR
git clone --single-branch git@codebase.helmholtz.cloud:openfoam/fwdc/Developments.git $WM_PROJECT_USER_DIR
```

#### 3.2.2. Source code from Rossendorf Data Repository (RODARE)

Download tar archive for the HZDR Multiphase Addon for OpenFOAM[^6] from RODARE
and unpack it into the OpenFOAM user directory

```shell
source $HOME/OpenFOAM/OpenFOAM-10-s.1/etc/bashrc
mkdir -p $WM_PROJECT_USER_DIR
tar -xzf HZDR-Multiphase-Addon-<version>.tgz -C $WM_PROJECT_USER_DIR
```

### 3.3 Compiling the Source Code

Firstly, apply the patches to the OpenFOAM installation provided with the
HZDR Multiphase Addon

```shell
source $HOME/OpenFOAM/OpenFOAM-10-s.1/etc/bashrc
cp $WM_PROJECT_USER_DIR/patches/*.sprint.tgz $WM_PROJECT_DIR
cd $WM_PROJECT_DIR
for f in *.sprint.tgz; do if [[ -f $f ]]; then tar xzf "$f"; fi; done
for f in *.sprint; do if [[ -f $f ]]; then git am "$f"; fi; done
rm -f *.sprint *.sprint.tgz
```

Secondly, compile the OpenFOAM installation by

```shell
cd $WM_PROJECT_DIR
./Allwmake -j
```

Once the compilation process is finished successfully, reload the OpenFOAM
environment and test the installation

```shell
source $HOME/OpenFOAM/OpenFOAM-10-s.1/etc/bashrc
foamVersion
```

For more details concerning other Linux operating systems, additional software
packages or more read the installation instructions for OpenFOAM-dev from Source
Repository[^9].

Finally, compile the HZDR Multiphase Addon

```shell
cd $WM_PROJECT_USER_DIR
./Allwmake -j
```

## 4. Option 3: Using Docker Images

Docker is a popular software for Operating-System-Level virtualisation and can
be used to deliver software in packages and a pre-defined environment. Those
packages are called Docker images and a running instance of such an image is
called a Docker container. Those containers are isolated from each other and
from the host system. For Helmholtz and Friends the HZDR Multiphase Addon for
OpenFOAM[^1] together with a pre-installed working environment for OpenFOAM can
be provided as Docker image. The Docker images in the corresponding Docker
registry[^1] feature support for graphical desktop applications, such as
Paraview. Supported are Intel and NVIDIA graphic cards, but currently no AMD
graphic cards.

**Optional:** A useful tool for defining, deploying and managing Docker
applications is `docker compose`. The HZDR Multiphase Addon for OpenFOAM[^1]
contains a set of docker compose files to build and launch the desired Docker
images.

### 4.1. Prerequisites

Follow the installation instructions for Docker and your operating system on the
Docker homepage[^10]. For security reasons, it is highly recommended to install
Docker in rootless mode for Linux[^11]. On Mac OS this is not required.

**Optional:** If the CUDA functionality of the HZDR Multiphase Addon for
OpenFOAM is of interest, the Nvidia Container Toolkit[^12] has to be installed.
Also, if `docker compose` is of interest, follow the installation instructions
for `docker compose V2+`[^14] to install it on Linux or enable
`Use Docker Compose V2` in Docker Desktop on Mac OS.

### 4.2. Available Docker Images at Helmholtz Code Base

The following table contains a list of available Docker images in the project's
docker registry[^1] including a pre-compiled OpenFOAM (`openfoam*`, recommended
for developers) or also a pre-compiled HZDR Multiphase Addon (`hzdr*`,
recommended for users).

| Docker image tag       | OpenFOAM           | HZDR Multiphase Addon | Display Server Support    | Cuda               | Compiler  | Compile Option |
|------------------------|--------------------|-----------------------|---------------------------|--------------------|-----------|----------------|
| hzdrLinuxGccOpt        | :white_check_mark: | :white_check_mark:    | :white_check_mark: Intel  | :x:                | Gcc       | Optimised      |
| hzdrNvidiaGccOpt       | :white_check_mark: | :white_check_mark:    | :white_check_mark: Nvidia | :x:                | Gcc       | Optimised      |
| hzdrCudaGccOpt         | :white_check_mark: | :white_check_mark:    | :x:                       | :white_check_mark: | Gcc, Nvcc | Optimised      |
| openfoamLinuxGccOpt    | :white_check_mark: | :x:                   | :white_check_mark: Intel  | :x:                | Gcc       | Optimised      |
| openfoamLinuxClangOpt  | :white_check_mark: | :x:                   | :white_check_mark: Intel  | :x:                | Clang     | Optimised      |
| openfoamLinuxGccDebug  | :white_check_mark: | :x:                   | :white_check_mark: Intel  | :x:                | Gcc       | Debugging      |
| openfoamNvidiaGccOpt   | :white_check_mark: | :x:                   | :white_check_mark: Nvidia | :x:                | Gcc       | Optimised      |
| openfoamNvidiaGccDebug | :white_check_mark: | :x:                   | :white_check_mark: Nvidia | :x:                | Gcc       | Debugging      |
| openfoamCudaGccOpt     | :white_check_mark: | :x:                   | :x:                       | :white_check_mark: | Gcc, Nvcc | Optimised      |

The source code of the OpenFOAM Foundation release is installed inside the
docker image in `$HOME/OpenFOAM/` and the HZDR Multiphase Addon for OpenFOAM is
located in `$WM_PROJECT_SITE`.

#### 4.2.1. Launch image with Docker

Login to docker registry

```shell
docker login registry.hzdr.de
```

and launch container

```shell
docker run -it registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<image> bash
```

or with CUDA support

```shell
docker run -it --gpus all registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<image> bash
```

Note, you have to replace `<image>` with an
[image tag](#42-available-docker-images-at-helmholtz-code-base).

#### 4.2.2. Launch image with foamDockerRun.sh

For a more comfortable usage a bash script `foamDockerRun.sh` is provided in the
repository[^1] at `bin/tools` to run an OpenFOAM Docker image and to provide
display support for X, XQuartz and XWayland on Intel and Nvidia graphic cards.

```shell
docker login registry.hzdr.de
```

```shell
# Display support for Intel
bin/tools/foamDockerRun.sh --desktop registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<image>
```

```shell
# Display support for Nvidia
bin/tools/foamDockerRun.sh --desktop --gpus all registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<image>
```

```shell
# No display support, but Cuda support
bin/tools/foamDockerRun.sh --gpus all registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<image>
```

For more information and options run `foamDockerRun.sh --help` or
`docker run --help`. Note, you have to replace `<image>` with an
[image tag](#42-available-docker-images-at-helmholtz-code-base).

#### 4.2.3. Launch image with `docker compose`

The pre-configured `docker-compose.yml` in the repository[^1] uses services to
run a pre-defined Docker environment. The following table contains a list of the
available services and the Docker images that are used. From within the
directory where you have cloned the repository, run

```shell
docker compose --profile '*' config --services
```

to get a full list of all available services.

| Service             | OpenFOAM           | HZDR Multiphase Addon | Display Server Support    | Cuda               | Compiler  | Compile Option |
|---------------------|--------------------|-----------------------|---------------------------|--------------------|-----------|----------------|
| hzdrLinux           | :white_check_mark: | :white_check_mark:    | :white_check_mark: Intel  | :x:                | Gcc       | Optimised      |
| hzdrNvidia          | :white_check_mark: | :white_check_mark:    | :white_check_mark: Nvidia | :x:                | Gcc       | Optimised      |
| hzdrCuda            | :white_check_mark: | :white_check_mark:    | :x:                       | :white_check_mark: | Gcc, Nvcc | Optimised      |
| openfoamLinux       | :white_check_mark: | :x:                   | :white_check_mark: Intel  | :x:                | Gcc       | Optimised      |
| openfoamLinuxDebug  | :white_check_mark: | :x:                   | :white_check_mark: Intel  | :x:                | Gcc       | Debugging      |
| openfoamLinuxProf   | :white_check_mark: | :x:                   | :white_check_mark: Intel  | :x:                | Gcc       | Profiling      |
| openfoamNvidia      | :white_check_mark: | :x:                   | :white_check_mark: Nvidia | :x:                | Gcc       | Optimised      |
| openfoamNvidiaDebug | :white_check_mark: | :x:                   | :white_check_mark: Nvidia | :x:                | Gcc       | Debugging      |
| openfoamNvidiaProf  | :white_check_mark: | :x:                   | :white_check_mark: Nvidia | :x:                | Gcc       | Profiling      |
| openfoamCuda        | :white_check_mark: | :x:                   | :x:                       | :white_check_mark: | Gcc, Nvcc | Optimised      |

To launch a service from the table above do

```shell
docker login registry.hzdr.de
```

```shell
# Launch service in detached mode (background)
docker compose up -d <service>
```

```shell
# execute command in running container
docker compose exec <service> bash
```

```shell
# stop service
docker compose down <service>
```

Note, you have to replace `<service>` with a
[service name](#423-launch-image-with-docker-compose). The source code of the
OpenFOAM Foundation release is installed inside the docker image in
`$HOME/OpenFOAM/` and the HZDR Multiphase Addon for OpenFOAM is located in
`$WM_PROJECT_SITE`.

### 4.3. Running Docker image from Rodare

This is currently not supported.

### 4.4. System and Git Configuration within Docker

#### 4.4.1. General Linux Configurations

Most software packages on Linux systems store their configurations in
`~/.config`. To persistently store and share configurations for software
packages, such as Paraview, this directory should be defined and created as a
Docker volume, and shared between Docker containers.

```shell
# create docker volume
docker volume create config
```

```shell
# mount volume when launching container
docker run -it --volume config:/root/.config <image> bash
```

When using `foamDockerRun.sh` this volume will be created and shared
automatically. When using `docker compose` the volume has to be created
manually, but will be shared automatically.

#### 4.4.2 Git Configuration

The Git version control system typically stores its configuration in
`~/.gitconfig`. It is recommended to copy the git configuration into the Docker
containers instead of binding the file as a volume to it. When using pure Docker
the following commands will copy the git configuration file into the container

```shell
CONTAINERID=$(docker create -it <image>)
docker cp $HOME/.gitconfig $CONTAINERID:/root/.gitconfig
docker start -i $CONTAINERID
```

For `docker compose` at least `docker compose V2+`[^14] is required, then use
the following commands

```shell
docker compose up -d <service>
```

```shell
docker compose cp $HOME/.gitconfig <service>:/root/.gitconfig
```

```shell
docker compose exec <service> bash
```

The script `foamDockerRun.sh` will copy the git configuration into the container
automatically. Note, you have to replace `<image>` or `<service>` with a
[image tag](#42-available-docker-images-at-helmholtz-code-base) or a
[service name](#423-launch-image-with-docker-compose), respectively.

## 5. Option 4: Building Docker Images

The required `Dockerfile` for building the Docker images with the required
OpenFOAM Foundation version and the HZDR Multiphase Addon for OpenFOAM is
available in `etc/docker`. The `Dockerfile` is a multi-stage `Dockerfile` with
the following two stages:

- **`openfoam`** - build Openfoam image without HZDR Multiphase Addon
- **`addon`** - build Openfoam image and HZDR Multiphase Addon

### 5.1. Building with Docker

For building the Docker images use the following command

```shell
docker build --target <openfoam|addon> --tag <image> --build-arg ... -f etc/docker/Dockerfile .
```

The available variables for `build-arg` are defined by the `Dockerfile` and the
default options are listed in the following table

| Name               | Default                                          | Options                                                              | Description                                |
|--------------------|--------------------------------------------------|----------------------------------------------------------------------|--------------------------------------------|
| FROMIMAGE          | ubuntu:20.04                                     | nvidia/opengl:base-ubuntu20.04, nvidia/cuda:11.2.0-devel-ubuntu20.04 | base image with operating system           |
| WM_PROJECT_VERSION | dev                                              | -                                                                    | OpenFOAM version, e.g. dev, 9, 10-s.1, ... |
| WM_COMPILE_OPTION  | Opt                                              | Debug, Prof                                                          | Compile Option for OpenFOAM                |
| WM_COMPILER        | Gcc                                              | Clang                                                                | Compiler for OpenFOAM                      |
| OFREPOSITORY       | <https://github.com/OpenFOAM/OpenFOAM-dev.git>   | -                                                                    | Git repository for OpenFOAM-dev            |
| TPREPOSITORY       | <https://github.com/OpenFOAM/ThirdParty-dev.git> | -                                                                    | Git repository for ThirdParty-dev          |
| WM_PROJECT_COMMIT  | Head                                             | -                                                                    | Valid Git commit sha for OpenFOAM-dev      |
| THIRDPARTY_COMMIT  | Head                                             | -                                                                    | Valid Git commit sha for ThirdParty-dev    |
| PARAVIEW           | No                                               | Yes                                                                  | Build Paraview                             |

Note, that in particular `WM_PROJECT_COMMIT` and `THIRDPARTY_COMMIT` have to be
set to the correct commit sha, otherwise the HZDR Multiphase Addon will not
build correctly. The most recent values can be found in `README.md` in the
repository[^1].

### 5.2. Building with Docker Compose

Building the images with `docker compose` is the most easy way and requires
almost no configuration. Use the following commands to build the Docker images
by yourself

```shell
# current sprint cycle release
docker compose -f docker-compose.yml -f etc/docker-compose/deployment.yml build <service>
```

```shell
# current OpenFOAM-dev release
docker compose --env-file etc/docker-compose/.env-dev build -f docker-compose.yml -f etc/docker-compose/deployment.yml <service>
```

### 5.3. Customised Docker image

The Docker images provide an OpenFOAM environment and the HZDR Multiphase Addon
for OpenFOAM. However, additional software like editors or graphical diff tools
are not installed as default to keep the size of the Docker images small. If
additional software packages are required, the Docker images can be customised.

#### 5.3.1. Customised Docker image with `docker commit`

The simplest way is to run the desired Docker image

```shell
docker login registry.hzdr.de
docker run -it registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<image> bash
```

and install all additional software packages inside the container

```shell
apt update
apt install ADD_UBUNTU_PACKAGE_HERE
```

Once all software packages are installed successfully, leave the container
running and open a new terminal. Use `docker ps` to extract the container id of
the running container and create a new image from it

```shell
docker commit <container id> registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<custom image>
```

Finally, stop the container and run the new custom image

```shell
docker run -it registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<custom image> bash
```

Note, you have to replace `<image>` with an
[image tag](#42-available-docker-images-at-helmholtz-code-base).

#### 5.3.2. Customised Dockerfile

The recommended way is to create your own `Dockerfile`[^15]. An example is shown
below

```docker
# Build variables required for docker image
ARG FROMIMAGE
FROM $FROMIMAGE

# Install custom ubuntu packages
RUN apt update \
    && apt install -qy --fix-missing --no-install-recommends \
       ADD_UBUNTU_PACKAGE_HERE \
       ADD_UBUNTU_PACKAGE_HERE \
       ADD_UBUNTU_PACKAGE_HERE

# Slim down packages
RUN apt clean autoclean && apt autoremove -y
```

To build your custom image with the selected software packages run

```shell
docker build --build-arg FROMIMAGE=registry.hzdr.de/openfoam/fwdc/developments/10-s.1:<image> .
```

Note, you have to replace `<image>` with an
[image tag](#42-available-docker-images-at-helmholtz-code-base).

#### 5.3.3. Customised docker-compose.override.yml

It is also possible to build a custom image with `docker compose`. Therefore, a
`Dockerfile` as shown above is required as well as a
`docker-compose.override.yml` in the project repository folder. A template can
be found in `etc/docker-compose`. Note, you have to replace `<image>` and
`<service>` with an
[image tag](#42-available-docker-images-at-helmholtz-code-base) and with a
[service name](#423-launch-image-with-docker-compose), respectively. Finally,
run the service

```shell
# Build customised docker image
docker compose build <service>
```

```shell
# Launch service in detached mode (background)
docker compose up -d <service>
```

```shell
# execute command in running container
docker compose exec <service> bash
```

For further information about the structure of the `docker-compose.override.yml`
and its syntax, refer to the general information about `docker compose`[^14] as
well as the compose specification [^13].

---
[^1]: https://codebase.helmholtz.cloud/openfoam/fwdc/Developments
[^2]: https://openfoam.org
[^3]: https://github.com/OpenFOAM/OpenFOAM-dev
[^4]: https://developer.nvidia.com/cuda-toolkit
[^5]: https://aai.helmholtz.de
[^6]: https://rodare.hzdr.de/record/767
[^7]: https://docs.gitlab.com/ee/user/ssh.html
[^8]: https://git-lfs.github.com
[^9]: https://openfoam.org/download/source/
[^10]: https://docs.docker.com/engine/installation
[^11]: https://docs.docker.com/engine/security/rootless
[^12]: https://github.com/NVIDIA/nvidia-docker
[^13]: https://docs.docker.com/compose/compose-file
[^14]: https://github.com/docker/compose
[^15]: https://docs.docker.com/engine/reference/builder
[^16]: https://support.apple.com/guide/disk-utility
